/*****************************************************************************
   callbacks.c - an Alsa based gtk mixer
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "main.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

extern GtkWidget *window;
extern Card *card; /* And array of the cards */
extern int cards; /* The number of cards in the system. */
extern Config config; /* The system config */

/* End Global Variables */
/*****************************************************************************/



void volume_adjust (GtkWidget *widget, MixerInfo *info)
{
  GtkWidget *adj;
  GtkWidget *button;
  char tip[64];
  int err;

  adj = button = widget;

  /* Take care of a volume change */
  if(info->flags & CHANNEL_LEFT || info->flags & CHANNEL_RIGHT)
    if(info->chan->flags & CHANNEL_SIMULTANEOUS) {
      info->chan->data.left = 
	info->chan->info.max - (int)GTK_ADJUSTMENT (adj)->value;
      info->chan->data.right = 
	info->chan->info.max - (int)GTK_ADJUSTMENT (adj)->value;
      GTK_ADJUSTMENT (info->other)->value = GTK_ADJUSTMENT (adj)->value;
      gtk_signal_emit_by_name (GTK_OBJECT (info->other), "changed");
      sprintf(tip, "%s Volume Left: %i", info->chan->info.name, 
	      info->chan->data.left);
      gtk_tooltips_set_tip(info->chan->left_tt, info->chan->lscal, tip, NULL);
      sprintf(tip, "%s Volume Right: %i", info->chan->info.name, 
	      info->chan->data.right);
      gtk_tooltips_set_tip(info->chan->right_tt, info->chan->rscal, tip, NULL);
    }
    else {
      if(info->flags & CHANNEL_LEFT) {
	info->chan->data.left = 
	  info->chan->info.max - (int)GTK_ADJUSTMENT (adj)->value;
	sprintf(tip, "%s Volume Left: %i", info->chan->info.name, 
		info->chan->data.left);
	gtk_tooltips_set_tip(info->chan->left_tt, info->chan->lscal, tip, NULL);
      }

      if(info->flags & CHANNEL_RIGHT) {
	info->chan->data.right = 
	  info->chan->info.max - (int)GTK_ADJUSTMENT (adj)->value;
	sprintf(tip, "%s Volume Right: %i", info->chan->info.name, 
		info->chan->data.right);
	gtk_tooltips_set_tip(info->chan->right_tt, 
			     info->chan->rscal, tip, NULL);
      }
    }

  if(info->flags & CHANNEL_MONO){
    info->chan->data.left = 
      info->chan->data.right = 
      info->chan->info.max - (int)GTK_ADJUSTMENT (adj)->value;
    sprintf(tip, "%s Volume Mono: %i", info->chan->info.name, 
	    info->chan->data.left);
    gtk_tooltips_set_tip(info->chan->mono_tt, info->chan->mscal, tip, NULL);
  }

  /* If either stereo mute button was pressed */
  if(info->flags & CHANNEL_MUTE_RIGHT || info->flags & CHANNEL_MUTE_LEFT)
    if(info->chan->flags & CHANNEL_SIMULTANEOUS)
      if(GTK_TOGGLE_BUTTON (button)->active) {
	info->chan->data.flags |= SND_MIXER_FLG_MUTE_RIGHT;
	info->chan->data.flags |= SND_MIXER_FLG_MUTE_LEFT;
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (info->other), TRUE);
	if(info->mute && info->unmute) {
	  gtk_widget_hide(info->unmute);
	  gtk_widget_show(info->mute);
	}
      }
      else {
	info->chan->data.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
	info->chan->data.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (info->other), FALSE);
	if(info->mute && info->unmute) {
	  gtk_widget_hide(info->mute);
	  gtk_widget_show(info->unmute);
	}
      }
    else {
      if(info->flags & CHANNEL_MUTE_RIGHT) 
	if(GTK_TOGGLE_BUTTON (button)->active) {
	  info->chan->data.flags |= SND_MIXER_FLG_MUTE_RIGHT;
	  if(info->mute && info->unmute) {
	    gtk_widget_hide(info->unmute);
	    gtk_widget_show(info->mute);
	  }
	}
	else {
	  info->chan->data.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
	  if(info->mute && info->unmute) {
	    gtk_widget_hide(info->mute);
	    gtk_widget_show(info->unmute);
	  }
	}
      if(info->flags & CHANNEL_MUTE_LEFT)
	if(GTK_TOGGLE_BUTTON (button)->active) {
	  info->chan->data.flags |= SND_MIXER_FLG_MUTE_LEFT;
	  if(info->mute && info->unmute) {
	    gtk_widget_hide(info->unmute);
	    gtk_widget_show(info->mute);	  
	  }
	}
	else {
	  info->chan->data.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
	  if(info->mute && info->unmute) {
	    gtk_widget_hide(info->mute);
	    gtk_widget_show(info->unmute);
	  }
	}
    }
  

  /* If a mono mute button was pressed */
  if(info->flags & CHANNEL_MUTE_MONO)
    if(GTK_TOGGLE_BUTTON (button)->active) {
      info->chan->data.flags |= (SND_MIXER_FLG_MUTE_RIGHT | 
				 SND_MIXER_FLG_MUTE_LEFT | 
				 SND_MIXER_FLG_MUTE);
      if(info->mute && info->unmute) {
	gtk_widget_hide(info->unmute);
	gtk_widget_show(info->mute);	  
      }
    }
    else {
      info->chan->data.flags &= ~(SND_MIXER_FLG_MUTE_RIGHT | 
				  SND_MIXER_FLG_MUTE_LEFT | 
				  SND_MIXER_FLG_MUTE);
      if(info->mute && info->unmute) {
	gtk_widget_hide(info->mute);
	gtk_widget_show(info->unmute);	  
      }
    }

  /* If a mono record button was pressed */
  if(info->flags & CHANNEL_RECORD)
    if(GTK_TOGGLE_BUTTON (button)->active) {
      info->chan->data.flags |= SND_MIXER_FLG_RECORD;
      if(info->mute && info->unmute) {
	gtk_widget_hide(info->unmute);
	gtk_widget_show(info->mute);
      }
    }
    else {
      info->chan->data.flags &= ~SND_MIXER_FLG_RECORD;
      if(info->mute && info->unmute) {
	gtk_widget_hide(info->mute);
	gtk_widget_show(info->unmute);
      }
    }

  /* If either of the stereo record buttons was pressed */
  if(info->flags & CHANNEL_RECORD_LEFT || info->flags & CHANNEL_RECORD_RIGHT) {
    if(info->chan->flags & CHANNEL_SIMULTANEOUS)
      if(GTK_TOGGLE_BUTTON (button)->active) {
	info->chan->data.flags |= SND_MIXER_FLG_RECORD_RIGHT;
	info->chan->data.flags |= SND_MIXER_FLG_RECORD_LEFT;
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (info->other), TRUE);
	if(info->mute && info->unmute) {
	  gtk_widget_hide(info->unmute);
	  gtk_widget_show(info->mute);
	}
      }
      else {
	info->chan->data.flags &= ~SND_MIXER_FLG_RECORD_RIGHT;
	info->chan->data.flags &= ~SND_MIXER_FLG_RECORD_LEFT;
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (info->other), FALSE);
	if(info->mute && info->unmute) {
	  gtk_widget_hide(info->mute);
	  gtk_widget_show(info->unmute);
	}
      }
    else {
      if(info->flags & CHANNEL_RECORD_RIGHT) 
	if(GTK_TOGGLE_BUTTON (button)->active) {
	  info->chan->data.flags |= SND_MIXER_FLG_RECORD_RIGHT;
	  if(info->mute && info->unmute) {
	    gtk_widget_hide(info->unmute);
	    gtk_widget_show(info->mute);
	  }
	}
	else {
	  info->chan->data.flags &= ~SND_MIXER_FLG_RECORD_RIGHT;
	  if(info->mute && info->unmute) {
	    gtk_widget_hide(info->mute);
	    gtk_widget_show(info->unmute);
	  }
	}
      if(info->flags & CHANNEL_RECORD_LEFT)
	if(GTK_TOGGLE_BUTTON (button)->active) {
	  info->chan->data.flags |= SND_MIXER_FLG_RECORD_LEFT;
	  if(info->mute && info->unmute) {
	    gtk_widget_hide(info->unmute);
	    gtk_widget_show(info->mute);	  
	  }
	}
	else {
	  info->chan->data.flags &= ~SND_MIXER_FLG_RECORD_LEFT;
	  if(info->mute && info->unmute) {
	    gtk_widget_hide(info->mute);
	    gtk_widget_show(info->unmute);
	  }
	}
    }
  }


  /* Nice and easy, if the simultaneous button is pressed */
  if(info->flags & CHANNEL_SIMULTANEOUS)
    if(GTK_TOGGLE_BUTTON (button)->active){
      info->chan->flags |= CHANNEL_SIMULTANEOUS;
      if(info->mute && info->unmute) {
	gtk_widget_hide(info->unmute);
	gtk_widget_show(info->mute);
      }
    }
    else {
      info->chan->flags &= ~CHANNEL_SIMULTANEOUS;
      if(info->mute && info->unmute) {
	gtk_widget_hide(info->mute);
	gtk_widget_show(info->unmute);
      }
    }

  if(info->flags & CHANNEL_LTOR)
    if(GTK_TOGGLE_BUTTON (info->other)->active) {
      info->chan->data.flags |= SND_MIXER_FLG_LTOR_IN;
    }
    else {
      info->chan->data.flags &= ~SND_MIXER_FLG_LTOR_IN;
    }
  if(info->flags & CHANNEL_RTOL)
    if(GTK_TOGGLE_BUTTON (info->other)->active) {
      info->chan->data.flags |= SND_MIXER_FLG_RTOL_IN;
    }
    else {
      info->chan->data.flags &= ~SND_MIXER_FLG_RTOL_IN;
    }
  

  /* Write whatever we've done to the kernel */
  err = snd_mixer_channel_write(info->mixer->handle, 
				info->channel, &info->chan->data);
  if(err < 0)
    printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));

}

void something_changed(gpointer data, int fd, GdkInputCondition condition)
{
  int err;
  snd_mixer_callbacks_t callbacks;
  Mixer *this_mixer = (Mixer *)data;

  /* Set up and install the status change callback */
  bzero(&callbacks, sizeof(callbacks));
  callbacks.private_data = (void *)this_mixer;
  callbacks.channel_was_changed = channel_changed;
  err = snd_mixer_read(this_mixer->handle, &callbacks);
  if(err < 0)
    printf("Error: %s", snd_strerror(err));
}

void channel_changed (void *private_data, int channel)
{
  Mixer *mxr = (Mixer *)private_data;
  int sim = 0;
  
  /* If the simultaneous button is set, undo the flag for a moment to not screw up independent actions */
  if(mxr->channel[channel].flags & CHANNEL_SIMULTANEOUS) {
    sim = 1;
    mxr->channel[channel].flags &= ~CHANNEL_SIMULTANEOUS;
  }

  /********************/
  /* Read in the data */
  snd_mixer_channel_read(mxr->handle, channel, &mxr->channel[channel].data);


  /***************************/
  /* Set the scales properly */
  /* Left */
  if(mxr->channel[channel].ladj) {
    GTK_ADJUSTMENT (mxr->channel[channel].ladj)->value = 
      (float) (mxr->channel[channel].info.max - mxr->channel[channel].data.left);
    gtk_signal_emit_by_name (GTK_OBJECT (mxr->channel[channel].ladj), 
			     "changed");
  }
  /* Right */
  if(mxr->channel[channel].ladj) {
    GTK_ADJUSTMENT (mxr->channel[channel].radj)->value = 
      (float) (mxr->channel[channel].info.max - mxr->channel[channel].data.right);
    gtk_signal_emit_by_name (GTK_OBJECT (mxr->channel[channel].radj), 
			     "changed");
  }

  /* Mono */
  if(mxr->channel[channel].madj) {
    GTK_ADJUSTMENT (mxr->channel[channel].madj)->value = 
      (float) (mxr->channel[channel].info.max - mxr->channel[channel].data.right);
    gtk_signal_emit_by_name (GTK_OBJECT (mxr->channel[channel].madj), 
			     "changed");
  }

  /******************/
  /* Set the muting */
  /* Left */
  if(mxr->channel[channel].lm)
    if (mxr->channel[channel].data.flags & SND_MIXER_FLG_MUTE_LEFT)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (mxr->channel[channel].lm), 
				  TRUE);
    else
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (mxr->channel[channel].lm), 
				  FALSE);

  /* Right */
  if(mxr->channel[channel].rm)
    if(mxr->channel[channel].data.flags & SND_MIXER_FLG_MUTE_RIGHT)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (mxr->channel[channel].rm), 
				  TRUE);
    else
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (mxr->channel[channel].rm), 
				  FALSE);
  /* Mono */
  if(mxr->channel[channel].mm)
    if(mxr->channel[channel].data.flags & SND_MIXER_FLG_MUTE || 
       mxr->channel[channel].data.flags & SND_MIXER_FLG_MUTE_LEFT ||
       mxr->channel[channel].data.flags & SND_MIXER_FLG_MUTE_RIGHT)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON
				  (mxr->channel[channel].mm),
				  TRUE);
    else
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON
				  (mxr->channel[channel].mm),
				  FALSE);
  
  /*******************/
  /* Set the Record */
  if(mxr->channel[channel].rec)
    if(mxr->channel[channel].data.flags & SND_MIXER_FLG_RECORD)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (mxr->channel[channel].rec), 
				  TRUE);
    else
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (mxr->channel[channel].rec), 
				  FALSE);

  if(mxr->channel[channel].lrec)
    if(mxr->channel[channel].data.flags & SND_MIXER_FLG_RECORD_LEFT)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (mxr->channel[channel].lrec), 
				  TRUE);
    else
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (mxr->channel[channel].lrec), 
				  FALSE);

  if(mxr->channel[channel].rrec)
    if(mxr->channel[channel].data.flags & SND_MIXER_FLG_RECORD_RIGHT)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (mxr->channel[channel].rrec), 
				  TRUE);
    else
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (mxr->channel[channel].rrec), 
				  FALSE);

  if(mxr->channel[channel].ltor_in)
    if(mxr->channel[channel].data.flags & SND_MIXER_FLG_LTOR_IN)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (mxr->channel[channel].ltor_in), 
				  TRUE);
    else
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (mxr->channel[channel].ltor_in), 
				  FALSE);

  if(mxr->channel[channel].rtol_in)
    if(mxr->channel[channel].data.flags & SND_MIXER_FLG_RTOL_IN)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (mxr->channel[channel].rtol_in), 
				  TRUE);
    else
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (mxr->channel[channel].rtol_in), 
				  FALSE);

  /* If the simultaneous button is set, reset the flag */
  if(sim)
    mxr->channel[channel].flags |= CHANNEL_SIMULTANEOUS;


  return;
}
