/*****************************************************************************
   cards.c - an Alsa based gtk mixer
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "main.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

extern Card *card; /* And array of the cards */
extern int cards; /* The number of cards in the system. */
extern Config config; /* The system config */

/* End Global Variables */
/*****************************************************************************/


int init_cards()
{
  /* Begin variable declarations */
  int i,j,k;
  int err;
  void *handle;
  snd_ctl_hw_info_t cardinfo; /* The info for the card */
  /* Begin variable declarations */
  
  /* Get the number of sound cards in the system */
  cards = snd_cards();
  
  /* Allocate out the handles */
  card = calloc(cards, sizeof(Card));
  
  
  
  
  
  /* Open up the mixers and get the mixer info. */
  for(i = 0; i < cards; i++) {
    /* Open the mixer */
    err = snd_ctl_open(&card[i].handle, i);
    if(err < 0) {
      card[i].handle = NULL;
      printf("Unable to open mixer for soundcard %i.  Error: %s\n", i, 
	     snd_strerror(err) );
    }
    /* Get the hardware information */
    if(card[i].handle) {
      err = snd_ctl_hw_info(card[i].handle, &cardinfo);
      if(err < 0) {
	printf("Unable to get control hardware info for soundcard %i.  Error: %s\n.", i, snd_strerror(err));
	/* If we can't get the soundcard's actual info, provide defaults */
	sprintf(card[i].name, "Sound Card #%i", i);
	card[i].nmixers = 1;
	card[i].npcms = 1;
      }
      else {
	card[i].nmixers = cardinfo.mixerdevs;
	card[i].npcms = cardinfo.pcmdevs;
	strcpy(card[i].name, cardinfo.name);
      }

      /* allocate the mixer array */
      card[i].mixer = calloc(card[i].nmixers, sizeof(Mixer));

      /* Get all of the appropriate information for each mixer */
      for(k = 0; k < card[i].nmixers; k++) {
	err = snd_mixer_open(&(card[i].mixer[k].handle), i, k);
	if(err < 0) {
	  card[i].handle = NULL;
	  printf("Unable to open mixer %i for soundcard %i.  Error: %s\n", 
		 i, k, snd_strerror(err) );
	}
	else {
	  /* Set exact mode */
	  err = snd_mixer_exact_mode(card[i].mixer[k].handle, TRUE);
	  if(err < 0) {
	    printf("Unable to switch mixer %i on card %i to exact mode.\n",
		   i, k);
	    printf("Error: %s\n", snd_strerror(err));
	    /* Some error handling should go here, such as being able to take this. */
	  }
	  
	  /* Set up the default values in the appropriate variables */
	  card[i].mixer[k].switch_table = NULL;
	  
	  err = snd_mixer_info(card[i].mixer[k].handle, &card[i].mixer[k].info);
	  if(err < 0) {
	    printf("Unable to get mixer info for mixer %i on sound card %i.\n",
		   i, k);
	    printf("Reason: %s.\n", snd_strerror(err));
	  }
	  else
	    /* Store the name of the mixer */
	    strcpy(card[i].mixer[k].name, card[i].mixer[k].info.name);

	
	  /* Time to get the channels */
	  /* First, get the number of channels. */
	  card[i].mixer[k].cnum = snd_mixer_channels(card[i].mixer[k].handle);
	  
	  /* Allocate the channels */
	  if(card[i].mixer[k].cnum > 0)
	    card[i].mixer[k].channel = calloc(card[i].mixer[k].cnum, 
					      sizeof(Channel));
	  else
	    card[i].mixer[k].channel = NULL;
	  
	  /* Now that we have someplace to put the info, put it there */
	  for(j = 0; j < card[i].mixer[k].cnum; j++) {
	    err = snd_mixer_channel_info(card[i].mixer[k].handle, 
					 j,
					 &card[i].mixer[k].channel[j].info);
	    err = snd_mixer_channel_read(card[i].mixer[k].handle, 
					 j,
					 &card[i].mixer[k].channel[j].data);
	    card[i].mixer[k].channel[j].num = j;
	    card[i].mixer[k].channel[j].mixer = &card[i].mixer[k];
	  }
	  
	  
	  card[i].mixer[k].snum = snd_mixer_switches(card[i].mixer[k].handle);
	  if(card[i].mixer[k].snum > 0)
	    card[i].mixer[k].switches = calloc(card[i].mixer[k].snum,
					       sizeof(Switch));
	  else
	    card[i].mixer[k].switches = NULL;
	  
	  for(j = 0; j < card[i].mixer[k].snum; j++) {
	    err = snd_mixer_switch_read(card[i].mixer[k].handle,
					j,
					&card[i].mixer[k].switches[j].data);
	    if(err < 0)
	      printf("Unable to get switch info for switch number %i.\n", j);
	  }
	  
	} /* Mixer conditional */
	
      } /* Mixer for loop */

      
    } /* conditional if the card was opened */
    
    
    
  } /* card for loop */
  
  return 1;
}
