/*****************************************************************************
   xamixer.c - an Alsa based gtk mixer
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "main.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

extern GtkWidget *window;
extern Card *card; /* And array of the cards */
extern int cards; /* The number of cards in the system. */
extern Config config; /* The system config */

/* End Global Variables */
/*****************************************************************************/

GtkWidget *create_options_page()
{
  GtkWidget *box, *tmpbox;
  GtkWidget *table;
  GtkWidget *button;
  GtkWidget *volume_entry;
  GtkWidget *label;
  GtkTooltips *tooltips;
  
  /* Create the storage widgets */
  box = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(box);
  table = gtk_table_new(2, 2, FALSE);
  gtk_widget_show(table);
  gtk_box_pack_start(GTK_BOX(box), table, FALSE, FALSE, 0);
  gtk_table_set_row_spacings(GTK_TABLE (table), 5);

  /**************************************/
  /* Create the actual config options */
  /**********************************/

  /***********************************/
  /* Create the switch hide option */
  button = gtk_check_button_new_with_label("Hide Mixer Switches.");
  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
  gtk_widget_show(button);
  gtk_table_attach_defaults(GTK_TABLE (table), button, 0, 1, 0, 1);
  gtk_signal_connect(GTK_OBJECT (button), "toggled", 
		     GTK_SIGNAL_FUNC (hide_switches), NULL);
  tooltips = gtk_tooltips_new();
  gtk_tooltips_set_tip(tooltips, button, "Toggle the displaying of special mixer switches.  This may not be applicable to everyone.", NULL);

  /**********************************/
  /* Create the set volume option */
  tmpbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(tmpbox);
  button = gtk_button_new_with_label("Set Volume");
  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
  gtk_widget_show(button);
  gtk_box_pack_start(GTK_BOX (tmpbox), button, FALSE, FALSE, 3);
  tooltips = gtk_tooltips_new();
  gtk_tooltips_set_tip(tooltips, button, 
		       "Set the volume on all channels to the indicated volume.",
		       NULL);
  volume_entry = gtk_entry_new_with_max_length(3);
  gtk_widget_set_usize(volume_entry, 40, -1);
  gtk_widget_show(volume_entry);
  gtk_box_pack_start(GTK_BOX (tmpbox), volume_entry, FALSE, FALSE, 0);
  tooltips = gtk_tooltips_new();
  gtk_tooltips_set_tip(tooltips, volume_entry, 
		       "Input a volume to set all channels to here.",
		       NULL);
  label = gtk_label_new("%");
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX (tmpbox), label, FALSE, FALSE, 0);
  gtk_table_attach_defaults(GTK_TABLE (table), tmpbox, 1, 2, 0, 1);
  gtk_signal_connect(GTK_OBJECT (button), "clicked", 
		     GTK_SIGNAL_FUNC (set_volumes), volume_entry);
  gtk_signal_connect(GTK_OBJECT (volume_entry), "activate", 
		     GTK_SIGNAL_FUNC (set_volumes), volume_entry);
  


  /* Make the switch hiding option button */
  tmpbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(tmpbox);
  gtk_table_attach_defaults(GTK_TABLE (table), tmpbox, 0, 1, 1, 2);
  button = gtk_button_new_with_label("Choose Channels");
  gtk_widget_show(button);
  gtk_box_pack_start(GTK_BOX (tmpbox), button, FALSE, FALSE, 0);
  tooltips = gtk_tooltips_new();
  gtk_tooltips_set_tip(tooltips, button, 
		       "Bring up the Channel Display Selection dialog", NULL);
  gtk_signal_connect(GTK_OBJECT (button), "clicked",
		     GTK_SIGNAL_FUNC (create_channel_display_dialog), NULL);



  tmpbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(tmpbox);
  gtk_table_attach_defaults(GTK_TABLE (table), tmpbox, 1, 2, 1, 2);
  button = gtk_button_new_with_label("Mute All");
  gtk_widget_show(button);
  gtk_box_pack_start(GTK_BOX (tmpbox), button, FALSE, FALSE, 0);
  tooltips = gtk_tooltips_new();
  gtk_tooltips_set_tip(tooltips, button, 
		       "Mute all channels on all mixers that support muting.",
		       NULL);
  gtk_signal_connect(GTK_OBJECT (button), "clicked", 
		     GTK_SIGNAL_FUNC (set_mute), NULL);
  button = gtk_button_new_with_label("unMute All");
  gtk_widget_show(button);
  gtk_box_pack_start(GTK_BOX (tmpbox), button, FALSE, FALSE, 0);
  tooltips = gtk_tooltips_new();
  gtk_tooltips_set_tip(tooltips, button, 
		       "unMute all channels on all mixers that support muting.",
		       NULL);
  gtk_signal_connect(GTK_OBJECT (button), "clicked", 
		     GTK_SIGNAL_FUNC (set_unmute), NULL);

  

  return box;
}


void hide_switches(GtkWidget *widget, gpointer data)
{
  int i,k;

  if(GTK_TOGGLE_BUTTON (widget)->active)
    if(config.flags & CONFIG_SWITCHES_HIDDEN)
      return;
    else {
      for(i = 0; i < cards; i++)
	for(k = 0; k < card[i].nmixers; k++)
	  if(card[i].mixer[k].switch_table)
	    gtk_widget_hide(card[i].mixer[k].switch_table);
      config.flags |= CONFIG_SWITCHES_HIDDEN;
    } /* end of if(config.flags & CONFIG_SWITCHES_HIDDEN) */
  else
    if(config.flags & CONFIG_SWITCHES_HIDDEN){
      for(i = 0; i < cards; i++)
	for(k = 0; k < card[i].nmixers; k++)
	  if(card[i].mixer[k].switch_table)
	    gtk_widget_show(card[i].mixer[k].switch_table);
      config.flags &= ~CONFIG_SWITCHES_HIDDEN;
    }
    else
      return;
}

void set_mute(GtkWidget *widget, gpointer data)
{
  int i,j,k;
  int err;

  /* Set all of the channels to mute */
  for(i = 0; i < cards; i++)
    for(k = 0; k < card[i].nmixers; k++)
      for(j = 0; j < card[i].mixer[k].cnum; j++) {
	if(card[i].mixer[k].channel[j].info.caps & SND_MIXER_CINFO_CAP_MUTE ||
	   card[i].mixer[k].channel[j].info.caps & SND_MIXER_CINFO_CAP_HWMUTE) {
	  card[i].mixer[k].channel[j].data.flags |= SND_MIXER_FLG_MUTE_RIGHT | 
	    SND_MIXER_FLG_MUTE_LEFT | 
	    SND_MIXER_FLG_MUTE ;

	  /* Now write the changes to the sound card */
	  err = snd_mixer_channel_write(card[i].mixer[k].handle, 
					j, &card[i].mixer[k].channel[j].data);
	  if(err < 0)
	  printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
	  channel_changed(&card[i].mixer[k], j);
	}
      } /* Channel for loop */
}

void set_unmute(GtkWidget *widget, gpointer data)
{
  int i,j,k;
  int err;

  /* Set all of the channels to mute */
  for(i = 0; i < cards; i++)
    for(k = 0; k < card[i].nmixers; k++)
      for(j = 0; j < card[i].mixer[k].cnum; j++) {
	if(card[i].mixer[k].channel[j].info.caps & SND_MIXER_CINFO_CAP_MUTE ||
	   card[i].mixer[k].channel[j].info.caps & SND_MIXER_CINFO_CAP_HWMUTE) {
	  card[i].mixer[k].channel[j].data.flags &= ~(SND_MIXER_FLG_MUTE_RIGHT |
						      SND_MIXER_FLG_MUTE_LEFT |
						      SND_MIXER_FLG_MUTE);

	  /* Now write the changes to the sound card */
	  err = snd_mixer_channel_write(card[i].mixer[k].handle, 
					j, &card[i].mixer[k].channel[j].data);
	  if(err < 0)
	  printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
	  channel_changed(&card[i].mixer[k], j);
	}
      } /* Channel for loop */
}


void set_volumes(GtkWidget *widget, gpointer data)
{
  int i,j,k;
  MixerInfo *info;
  int percent_vol;
  int vol;

  /* Get the volume */
  percent_vol = atoi(gtk_entry_get_text(GTK_ENTRY (data)));

  /* Set all of the volumes */
  /* We'll have to do the setting here, because of the percentage conversion */
  for(i = 0; i < cards; i++)
    for(k = 0; k < card[i].nmixers; k++)
      for(j = 0; j < card[i].mixer[k].cnum; j++) {
	/* Convert the percentage volume to actual volume */
	vol = (percent_vol * card[i].mixer[k].channel[j].info.max) / 100;

	/* Left */
	if(card[i].mixer[k].channel[j].ladj) {
	  GTK_ADJUSTMENT (card[i].mixer[k].channel[j].ladj)->value = 
	    (float) (card[i].mixer[k].channel[j].info.max - vol);
	  gtk_signal_emit_by_name (GTK_OBJECT 
				   (card[i].mixer[k].channel[j].ladj), 
				   "changed");
	  gtk_signal_emit_by_name (GTK_OBJECT 
				   (card[i].mixer[k].channel[j].ladj), 
				   "value_changed");
	}
	/* Right */
	if(card[i].mixer[k].channel[j].ladj) {
	  GTK_ADJUSTMENT (card[i].mixer[k].channel[j].radj)->value = 
	    (float) (card[i].mixer[k].channel[j].info.max - vol);
	  gtk_signal_emit_by_name (GTK_OBJECT 
				   (card[i].mixer[k].channel[j].radj), 
				   "changed");
	  gtk_signal_emit_by_name (GTK_OBJECT 
				   (card[i].mixer[k].channel[j].radj), 
				   "value_changed");
	}
	
	/* Mono */
	if(card[i].mixer[k].channel[j].madj) {
	  GTK_ADJUSTMENT (card[i].mixer[k].channel[j].madj)->value = 
	    (float) (card[i].mixer[k].channel[j].info.max - vol);
	  gtk_signal_emit_by_name (GTK_OBJECT 
				   (card[i].mixer[k].channel[j].madj), 
				   "changed");
	  gtk_signal_emit_by_name (GTK_OBJECT 
				   (card[i].mixer[k].channel[j].madj), 
				   "value_changed");
	}
	
      } /* end of channel for loop */
  
}

void create_channel_display_dialog(GtkWidget *widget, gpointer data)
{
  GtkWidget *win;
  GtkWidget *check;
  GtkWidget *label;
  GtkWidget *frame;
  GtkWidget *mainbox, *box;
  GtkWidget *table;
  int i,k,j,ctotal,cnum,t,w=2;
  char name[128];

  if(config.cdisplay) {
    gtk_widget_show(config.cdisplay);
    return;
  }

  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW (win), "Channel Display Options");
  mainbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(mainbox);
  gtk_container_add(GTK_CONTAINER (win), mainbox);
  gtk_signal_connect(GTK_OBJECT (win), "delete_event", 
		     GTK_SIGNAL_FUNC (gtk_widget_hide), NULL);

  /* Get the number of channels to be displayed */
  ctotal = 0;
  for(i = 0; i < cards; i++)
    for(k = 0; k < card[i].nmixers; k++)
      ctotal += card[i].mixer[k].cnum;

  /* The width will be the cube root of the total 
   */
  i = 0;
  while(i*i < ctotal)
    w = i++;

  for(i = 0; i < cards; i++)
    for(k = 0; k < card[i].nmixers; k++) {
      cnum = 0;
      sprintf(name, "Card %i (%s), Mixer %i (%s)", i, card[i].name, 
	      k, card[i].mixer[k].info.name);
      frame = gtk_frame_new(name);
      gtk_widget_show(frame);
      gtk_container_border_width(GTK_CONTAINER (frame), 5);
      gtk_box_pack_start(GTK_BOX (mainbox), frame, FALSE, FALSE, 0);
      table = gtk_table_new(w, (card[i].mixer[k].cnum + (w-1))/ w, FALSE);
      gtk_widget_show(table);
      gtk_container_add(GTK_CONTAINER (frame), table);
      gtk_table_set_row_spacings(GTK_TABLE (table), 5);

      for(t = 0; t < w; t++)
	for(j = 0; 
	    j < ((card[i].mixer[k].cnum + (w-1))/w) && 
	      (t*((card[i].mixer[k].cnum + (w-1))/w))+j < card[i].mixer[k].cnum;
	    j++) {
	  check = 
	    gtk_check_button_new_with_label(card[i].mixer[k].channel[cnum].info.name);
	  gtk_button_set_relief (GTK_BUTTON (check), GTK_RELIEF_NONE);
	  /* (t*((card[i].mixer[k].cnum + (w-1))/w))+j */
	  gtk_widget_show(check);
	  gtk_table_attach_defaults(GTK_TABLE (table), check, t, t+1, j, j+1);

	  if(card[i].mixer[k].channel[j].flags & CHANNEL_DISPLAYED)
	    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (check), TRUE);

	  gtk_signal_connect(GTK_OBJECT (check), "clicked", 
			     GTK_SIGNAL_FUNC (channel_display_toggle), 
			     &card[i].mixer[k].channel[cnum]);
	  cnum++; /* keep track of the channel number */
	} /* end of channel loop */
      
  
    } /* End of mixer loop */

  gtk_widget_show(win);
  config.cdisplay = win;
}

void channel_display_toggle(GtkWidget *widget, Channel *chan)
{
  if(GTK_TOGGLE_BUTTON (widget)->active) {
    if(chan->flags & CHANNEL_DISPLAYED)
      return;
    chan->flags |= CHANNEL_DISPLAYED;

/*     if(chan->num < (MIXER (chan->mixer)->cnum)-1) */
/*       gtk_table_set_col_spacing(GTK_TABLE (chan->label->parent), chan->num, 5); */

    if(chan->lm)
      gtk_widget_show(chan->lm); 
    if(chan->rm)
      gtk_widget_show(chan->rm); 
    if(chan->mm)
      gtk_widget_show(chan->mm); 
    if(chan->rec)
      gtk_widget_show(chan->rec); 
    if(chan->lrec)
      gtk_widget_show(chan->lrec); 
    if(chan->rrec)
      gtk_widget_show(chan->rrec);
    if(chan->lscal)
      gtk_widget_show(chan->lscal); 
    if(chan->rscal)
      gtk_widget_show(chan->rscal); 
    if(chan->mscal)
      gtk_widget_show(chan->mscal);
    if(chan->label)
      gtk_widget_show(chan->label);
    if(chan->lock)
      gtk_widget_show(chan->lock);
  }
  else {
    if(!(chan->flags & CHANNEL_DISPLAYED))
      return;
    chan->flags &= ~CHANNEL_DISPLAYED;
    
    if(chan->lm)
      gtk_widget_hide(chan->lm); 
    if(chan->rm)
      gtk_widget_hide(chan->rm); 
    if(chan->mm)
      gtk_widget_hide(chan->mm); 
    if(chan->rec)
      gtk_widget_hide(chan->rec); 
    if(chan->lrec)
      gtk_widget_hide(chan->lrec); 
    if(chan->rrec)
      gtk_widget_hide(chan->rrec);
    if(chan->lscal)
      gtk_widget_hide(chan->lscal); 
    if(chan->rscal)
      gtk_widget_hide(chan->rscal); 
    if(chan->mscal)
      gtk_widget_hide(chan->mscal);
    if(chan->label)
      gtk_widget_hide(chan->label);
    if(chan->lock)
      gtk_widget_hide(chan->lock);

/*     if(chan->num < (MIXER (chan->mixer)->cnum - 1)) */
/*       gtk_table_set_col_spacing(GTK_TABLE (chan->label->parent), chan->num, 0); */
  }

  return;
}
