/*****************************************************************************
   xamixer.c - an Alsa based gtk mixer
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "main.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

extern GtkWidget *window;
extern Card *card; /* And array of the cards */
extern int cards; /* The number of cards in the system. */
extern Config config; /* The system config */

/* End Global Variables */
/*****************************************************************************/

GtkWidget *create_switch_dialog(GtkWidget *widget, gpointer data)
{
  Mixer *mixer = (Mixer *)data;
  GtkWidget *win;
  GtkWidget *table;
  GtkWidget *box;
  GtkWidget *button;
  GtkWidget *entry;
  GtkWidget *label;
  GtkTooltips *tooltips;
  MixerInfo *info;
  int i,j,k=2, snum;
  char text[64];
  char tip[256];

  /* If the mixer being passed doesn't actually exist, don't fuck things up */
  if(!mixer)
    return;

  /* If the dialog already exists, why make it again? */
  if(mixer->switch_table) {
    gtk_widget_show(mixer->switch_table);
    return;
  }


  /* Make the container table */
  table = gtk_table_new((mixer->snum + 1)/k, k, FALSE);
  gtk_widget_show(table);

  /* Why create a switch dialog for no switches */
  if(mixer->snum <= 0)
    return table;


  if(k > mixer->snum)
    k = mixer->snum;
  for(i=0; i < k; i++) {
    for(j = 0; j < ((mixer->snum + 1)/k); j++) {
      /* get the number of the switch */
      snum = i*((mixer->snum + 1)/k) + j;
      if(snum >= mixer->snum)
	continue;
      
      /* Create and attach the box */
      box = gtk_hbox_new(FALSE, 0);
      gtk_widget_show(box);
      gtk_table_attach_defaults(GTK_TABLE(table), box, i, i+1, j, j+1);

      switch(mixer->switches[snum].data.type) {
      case SND_MIXER_SW_TYPE_BOOLEAN:
	button = gtk_check_button_new();
	gtk_widget_show(button);
	gtk_box_pack_start(GTK_BOX (box), button, FALSE, FALSE, 0);
	tooltips = gtk_tooltips_new();
	gtk_tooltips_set_tip(tooltips, button, "Toggle On\\Off", NULL);
	if(mixer->switches[snum].data.value.enable)
	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
	else
	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), FALSE);
	info = create_mixer_info(mixer, snum, 0);
	gtk_signal_connect(GTK_OBJECT (button), "clicked",
			  GTK_SIGNAL_FUNC(change_switch), info);
	break;
      case SND_MIXER_SW_TYPE_BYTE:
	entry = gtk_entry_new_with_max_length(3);
	gtk_widget_show(entry);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, FALSE, 0);
	info = create_mixer_info(mixer, snum, 0);
	tooltips = gtk_tooltips_new();
	sprintf(tip, "8bit range: %i - %i", mixer->switches[snum].data.low,
		mixer->switches[snum].data.high);
	gtk_tooltips_set_tip(tooltips, button, tip, NULL);

	gtk_signal_connect(GTK_OBJECT (button), "clicked",
			  GTK_SIGNAL_FUNC(change_switch), info);
	break;
      case SND_MIXER_SW_TYPE_WORD:
	entry = gtk_entry_new_with_max_length(5);
	gtk_widget_show(entry);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, FALSE, 0);
	info = create_mixer_info(mixer, snum, 0);
	tooltips = gtk_tooltips_new();
	sprintf(tip, "16bit range: %i - %i", mixer->switches[snum].data.low,
		mixer->switches[snum].data.high);
	gtk_tooltips_set_tip(tooltips, button, tip, NULL);
	gtk_signal_connect(GTK_OBJECT (button), "clicked",
			  GTK_SIGNAL_FUNC(change_switch), info);
	break;
      case SND_MIXER_SW_TYPE_DWORD:
	entry = gtk_entry_new_with_max_length(10);
	gtk_widget_show(entry);
	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, FALSE, 0);
	info = create_mixer_info(mixer, snum, 0);
	tooltips = gtk_tooltips_new();
	sprintf(tip, "32bit range: %i - %i", mixer->switches[snum].data.low,
		mixer->switches[snum].data.high);
	gtk_tooltips_set_tip(tooltips, button, tip, NULL);
	gtk_signal_connect(GTK_OBJECT (button), "clicked",
			  GTK_SIGNAL_FUNC(change_switch), info);
	break;
      default:
	break;
      }
      if(mixer->switches[snum].data.type == SND_MIXER_SW_TYPE_BOOLEAN)
	sprintf(text, "%s", mixer->switches[snum].data.name);
      else
	sprintf(text, "%s (%i - %i)", mixer->switches[snum].data.name, 
		mixer->switches[snum].data.low, 
		mixer->switches[snum].data.high);

      label = gtk_label_new(text);
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX (box), label, FALSE, FALSE, 0);


    } /* The column for loop */
  } /* the row for loop */


  mixer->switch_table = table;
  return table;
}


void change_switch(GtkWidget *widget, gpointer data)
{
  MixerInfo *info = (MixerInfo *)data;
  Switch *sw = &info->mixer->switches[info->channel];
  int err;

  switch(sw->data.type) {
  case SND_MIXER_SW_TYPE_BOOLEAN:
    sw->data.value.enable = GTK_TOGGLE_BUTTON (widget)->active;
    err = snd_mixer_switch_write(info->mixer->handle, info->channel, &sw->data);
    if(err < 0) {
      printf("Unable to write switch switch to Mixer.\n");
      printf("Reason: %s.\n", snd_strerror(err));
    }
    break;
    /* The other switch types should go here */
  default:
    break;
  }



  return;
}








