/*****************************************************************************
   xamixer.c - an Alsa based gtk mixer
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "main.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

GtkWidget *window;
Card *card; /* And array of the cards */
int cards; /* The number of cards in the system. */
extern Config config; /* The system config */

/* End Global Variables */
/*****************************************************************************/

int main(int argc, char **argv)
{
	/* Begin Variable Declarations */
	GtkWidget *mainbox;
	GtkWidget *notebook;
	GtkWidget *frame;
	GtkWidget *label;
	GtkWidget *table;
	GtkWidget *switch_button;
	GtkWidget *tmpbox;
	GtkWidget *tablebox;
	GtkWidget *separator;
	int i,j,k,xpm,found,fd;
	char title[32];
	char name[128];
	ChannelLabel *tmp;
	char labelname[256];
	char *home_env, *home_dir;
	GtkStyle *style;
	GtkWidget *hbox;
	/* End Variable Declarations */

  /* Go through gtk initialization */
	gtk_init(&argc, &argv);

	home_env = getenv("HOME");
	home_dir = calloc((strlen(home_env) + 2 + strlen(RCFILE)), 1);
	strcpy(home_dir, home_env);
	strcat(home_dir, "/");
	strcat(home_dir, RCFILE);
	gtk_rc_parse(home_dir);
	free(home_dir);

  /* Read in the soundcard info */
	if(!init_cards()) {
		printf("Error.  Unable to initialize sound cards.\n");
		return 1;
	}

	/* Read in normal config info */
	config_init();
	config_read("/usr/local/etc/xamixer.conf");
	home_env = getenv("HOME");
	home_dir = calloc((strlen(home_env) + 2 + strlen(HOME_FILE)), 1);
	strcpy(home_dir, home_env);
	strcat(home_dir, "/");
	strcat(home_dir, HOME_FILE);
	config_read(home_dir);
	free(home_dir);
  
	/* Make the title */
	sprintf(title, "XAmixer %s", VERSION);

	/* Create the main window */
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), title);
	gtk_signal_connect(GTK_OBJECT (window), "delete_event", 
			   (GtkSignalFunc) gtk_main_quit, NULL);
	signal(SIGINT, (void *)gtk_main_quit);
	/* Set the policy */
	gtk_window_set_policy(GTK_WINDOW (window), TRUE, TRUE, TRUE);
	/* Set the position, if one has been defined */
	gtk_widget_set_uposition(window, config.x_pos, config.y_pos);
	/* Realize the window so that we can start drawing pixmaps to it */
	gtk_widget_realize(window);

  /* Set up the pixmaps */
	setup_pixmaps(window);

	/* Set up the background pixmap */
	if(config.flags & CONFIG_BACKGROUND_XPM && 
	   config.flags & CONFIG_USE_XPMS) {
		style = gtk_widget_get_style(window);
		style->bg_pixmap[0] = gdk_pixmap_ref(config.background_xpm);
		style->bg_pixmap[1] = gdk_pixmap_ref(config.background_xpm);
		style->bg_pixmap[2] = gdk_pixmap_ref(config.background_xpm);
		style->bg_pixmap[3] = gdk_pixmap_ref(config.background_xpm);
		style->bg_pixmap[4] = gdk_pixmap_ref(config.background_xpm);
	}

	/* Create the notebook */
	notebook = gtk_notebook_new();
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
	/*   gtk_box_pack_start(GTK_BOX (mainbox), notebook, FALSE, FALSE, 0); */
	gtk_widget_show(notebook);
	gtk_container_add(GTK_CONTAINER(window), notebook);
  
	/* Create the notebook pages */
	for(i = 0; i < cards; i++) {
		for(k = 0; k < card[i].nmixers; k++) {
			frame = gtk_frame_new("");
			gtk_widget_show(frame);
			tablebox = gtk_vbox_new(FALSE, 0);
			gtk_widget_show(tablebox);
			gtk_container_add(GTK_CONTAINER(frame), tablebox);
			table = gtk_table_new(8, card[i].mixer[k].cnum, FALSE);
			/*       gtk_table_set_col_spacings(GTK_TABLE (table), config.padding); */
			gtk_box_pack_start(GTK_BOX (tablebox), table, FALSE, FALSE, 0);
			gtk_widget_show(table);

      
			for(j = 0; j < card[i].mixer[k].cnum; j++) {
				tmp = config.labels;
				found = xpm = 0;
				EAZERO(name, 128);
				while(tmp != NULL)
					if(is_same(card[i].mixer[k].channel[j].info.name, tmp->channel)) {
						found = 1;
						strncpy(name, tmp->label, 127);
						tmp = NULL;
					}
					else
						tmp = tmp->next;
	
				tmp = config.xpm;
				while(tmp != NULL)
					if(is_same(card[i].mixer[k].channel[j].info.name, tmp->channel)) {
						fd=open(tmp->label, O_RDONLY);
						if(fd != -1) {
							close(fd);
							found = xpm = 1;
							strncpy(name, tmp->label, 127);
						}
						else
							printf("Unable to open pixmap %s.\n", tmp->label);
						tmp = NULL;
					}
					else
						tmp = tmp->next;
	
				if(!found)
					strncpy(name, card[i].mixer[k].channel[j].info.name, 127);
	
				create_volume_control(name,
						      xpm,
						      card[i].mixer[k].channel[j].data.left,
						      card[i].mixer[k].channel[j].data.right,
						      &card[i].mixer[k],
						      j,
						      &card[i].mixer[k].channel[j].data,
						      &card[i].mixer[k].channel[j].info,
						      &card[i].mixer[k].channel[j],
						      window,
						      table);
	
			}

			/* Add in a separator for aesthetic effect */
			separator = gtk_hseparator_new();
			gtk_widget_show(separator);
			gtk_box_pack_start(GTK_BOX (tablebox), separator, FALSE, FALSE, 0);
      
			/* Add the switches in */
			if(card[i].mixer[k].snum >= 0)
				gtk_box_pack_start(GTK_BOX (tablebox), 
						   create_switch_dialog(table, &card[i].mixer[k]), 
						   FALSE, FALSE, 0);
			else
				card[i].mixer[k].switch_table = NULL;

			/* Create the label and add the page to the notebook */
			bzero(labelname, 256);
			if(config.flags & CONFIG_SHOW_CARD_NAME) {
				strcpy(labelname, card[i].name);
				if(config.flags & (CONFIG_SHOW_MIXER_NAME | CONFIG_SHOW_MIXER_NUMBER))
					strcat(labelname, ", ");
			}

			if(config.flags & CONFIG_SHOW_MIXER_NUMBER) {
				/* Do some trickery to get around an additional variable, plus this may be mroe efficient, 
				   since strcat() has to figure out where the end of the line is anyhow, plus the copying. */
				sprintf(&labelname[strlen(labelname)], "Mixer %i", k);
				if(config.flags & CONFIG_SHOW_MIXER_NAME)
					strcat(labelname, ", ");
			}
      
			if(config.flags & CONFIG_SHOW_MIXER_NAME)
				strcat(labelname, card[i].mixer[k].name);

			/* Just in case nothing is specified in the config file */
			if(!(config.flags & (CONFIG_SHOW_CARD_NAME | 
					     CONFIG_SHOW_MIXER_NAME | CONFIG_SHOW_MIXER_NUMBER)))
				sprintf(labelname, "%i", i + k);

			label = gtk_label_new(labelname);
			gtk_widget_show(label);
			gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);
      
		}

	}

	/* Create the options page */
	frame = gtk_frame_new("");
	gtk_widget_show(frame);
	gtk_container_add(GTK_CONTAINER (frame), create_options_page());
	label = gtk_label_new("Options");
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK (notebook), frame, label);
  
  /* Set the up select() on the file descriptors to update upon changes */
	for (i = 0; i < cards; i++)
		for(k = 0; k < card[i].nmixers; k++)
			gdk_input_add(snd_mixer_file_descriptor(card[i].mixer[k].handle),
				      GDK_INPUT_READ,
				      something_changed,
				      &card[i].mixer[k]);
  

	/* Set up the icon, if one has been defined. */
	if(config.flags & CONFIG_ICON_XPM && config.icon_xpm)
		gdk_window_set_icon(window->window, NULL, 
				    config.icon_xpm, config.icon_mask);

  

  
  
	/* Show the window */
	gtk_widget_show(window);


	/* And go to the main event loop */
	gtk_main();




	return 0;
}

















